#ifndef __CF_BYTEARRAY_H__
#define __CF_BYTEARRAY_H__

/**
 * \file 字节数组
 */

#include <Core/CFTypes.h>
#include <Core/CFList.h>

/**
 * \brief CFByteArray 字节数组.
 *
 * CFByteArray 字节数组类，用于存储一组字节数据；
 */
class CFByteArray {

public:
    /**
     * 默认构造函数.
     */ 
    CFByteArray();

    /**
     * 构造函数。
     * 
     * 构造一个包含str前size个字节的字节数组。
     * 
     * 如果str为空指针，则会构造一个空的CFByteArray。
     *
     * 如果size未被指定，str会被认定为以'\0'结尾的字符串，而且它的长度可以被动态确定。
     * 
     * CFByteArray 会对str进行深拷贝。
     *
     * @param [in] str 需要用来构造CFByteArray的数据。
     * @param [in] size str的长度。
     * @see CFByteArray(const CFUChar* str, int size = -1);
     */
    CFByteArray(const CFChar* str, int size = -1);

    /**
     * 构造函数。
     * 
     * 构造一个包含str前size个字节的字节数组。
     * 
     * 如果str为空指针，则会构造一个空的CFByteArray。
     *
     * 如果size未被指定，str会被认定为以'\0'结尾的字符串，而且它的长度可以被动态确定。
     * 
     * CFByteArray 会对str进行深拷贝。
     *
     * @param [in] str 需要用来构造CFByteArray的数据。
     * @param [in] size str的长度。
     * @see CFByteArray(const CFChar* str, int size = -1);
     */
    CFByteArray(const CFUChar* str, int size = -1);

    /**
     * 拷贝构造函数.
     * @param [in] other 另一个CFByteArray类实例。
     */
    CFByteArray(const CFByteArray& other);

    /**
     * 析构函数.
     * 在CFByteArray被销毁时会被自动调用。
     * 在析构函数中进行内存释放等操作。
     */
    virtual ~CFByteArray();

    /**
     * 获取字节数组中数据的长度。
     *
     * 获取字节数组中数据的长度，即所包含数据的字节数。
     *
     * @return CFByteArray中数据的字节长度。
     */
    CFUInt size();

    /**
     * 获取字节数组中指向数据的指针。
     *
     * 返回字节数组中一个指向数据的指针，该指针可以对数据进行读取与修改。
     *
     * 该指针所指向的数据以'\0'结尾。
     *
     * @return 返回字节数组中一个指向数据的指针。
     */
    CFUChar* data();

    /**
     * 获取字节数组中指向数据的指针。
     *
     * 返回字节数组中一个指向数据的指针，该指针可以对数据进行读取，但无法修改数据。
     *
     * 该指针所指向的数据以'\0'结尾。
     *
     * @return 返回字节数组中一个指向数据的指针。
     */
    const CFUChar* data() const;

    /**
     * 使用sep对CFByteArray进行分割。
     *
     * 使用sep来将CFByteArray分割成多个子数组，并生成一个子数组list返回给调用者。
     * 
     * 若CFByteArray中不包含sep，则会返回只有一个元素的list。
     *
     * @param [in] sep 用以分割CFByteArray的分割符。
     * @return 通过sep分割后生成的CFByteArray列表。
     */
    CFList<CFByteArray> split(CFUChar sep);

    /**
     * CFByteArray截取。
     *
     * 从 pos 位置开始截取 len 个字符，并将该CFByteArray返回给调用者。
     *
     * @param [in] pos 需要截取CFByteArray的起始位置，范围：0～size()。
     * @param [in] len 需要截取CFByteArray的长度，若需要截取 pos 后的全部CFByteArray，则传入 -1 .
     * @return 截取操作后的CFByteArray。
     */
    CFByteArray sub(CFUInt pos, CFInt len);

    /**
     * CFByteArray查找。
     *
     * 从 from 位置开始查找 chr 字符，caseSensitive参数表示是否区分大小写。
     * @param [in] chr 需要查找的字符。
     * @param [in] from 起始位置，默认为0。
     * @param [in] caseSensitive 是否区分大小写，区分大小写为true，否则为false，默认区分大小写.
     * @return 返回chr字符第一次找到的位置，返回-1表示未找到chr。
     */
    CFInt indexOf(CFUChar chr, int from = 0, bool caseSensitive = true);

    /**
     * CFByteArray查找。
     *
     * 从 from 位置开始查找 str CFByteArray，caseSensitive参数表示是否区分大小写。
     * @param [in] str 需要查找的CFByteArray。
     * @param [in] from 起始位置，默认为0。
     * @param [in] caseSensitive 是否区分大小写，区分大小写为true，否则为false，默认区分大小写.
     * @return 返回strCFByteArray第一次找到的位置，返回-1表示未找到str。
     */
    CFInt indexOf(const CFByteArray& str, int from = 0, bool caseSensitive = true);

    /**
     * 判断是否为空CFByteArray.
     *
     * @return 是否为空CFByteArray，true为空CFByteArray，否则为非空CFByteArray。
     */
    bool isEmpty() const;

    /**
     * 判断CFByteArray是否与 other CFByteArray相同.
     *
     * @param [in] other 需要与之比较的CFByteArray。
     * @return 是否与 other CFByteArray相同，true为相同，否则为不同。
     * @see compare()
     */
    bool equal(const CFByteArray& other);

    /**
     * 比较两个CFByteArray。
     *
     * @param [in] other 需要与之比较的CFByteArray。
     * @return 0 为与otherCFByteArray相同。
     * @return >0 则CFByteArray大于 other CFByteArray。
     * @return <0 则CFByteArray小于 other CFByteArray。
     * @see equal()
     */
    CFInt compare(const CFByteArray& other);

    void display();

public:
    friend CFByteArray operator+(const CFByteArray& str1, const CFByteArray& str2);
    friend CFByteArray operator+(const CFByteArray& str1, const CFUChar* str2);
    friend CFByteArray operator+(const CFUChar* str1, const CFByteArray& str2);
    friend CFByteArray operator+(const CFByteArray& str1, const CFChar* str2);
    friend CFByteArray operator+(const CFChar* str1, const CFByteArray& str2);
    // friend CFByteArray operator+(const CFUChar* str1, const CFUChar* str2);

    CFByteArray& operator=(const CFByteArray& str1);
    CFByteArray& operator+=(const CFByteArray& str1);

    bool operator==(const CFByteArray& other);
    bool operator!=(const CFByteArray& other);
    bool operator>(const CFByteArray& other);
    bool operator>=(const CFByteArray& other);
    bool operator<(const CFByteArray& other);
    bool operator<=(const CFByteArray& other);

private:
    CFUInt getUTF8length();
    void create(const CFUChar* strdata, int size = -1);

    CFUInt dataLength(const CFUChar* strdata);
    bool charEqual(CFUChar destChr, CFUChar srcChr, bool caseSensitive = true);

private:
    CFUChar* m_data;
    CFUInt m_strLen;
};

#endif // __CF_BYTEARRAY_H__
