#ifndef __CF_LIST_H__
#define __CF_LIST_H__

#include <Core/CFTypes.h>

#include <vector>

/**
 * \brief CFList 容器类.
 *
 * CFList是一个模版类，用于储存同一种类型数据的一个集合；
 *    提供方便接口函数，让使用者可以方便进行数据读取、写入、遍历、删除、清除等操作。
 */
template<typename T>
class CFList {
public:
    /**
     * 默认构造函数.
     */
    CFList()
        : m_vector(std::vector<T>()) {

    }
    
    /**
     * 拷贝构造函数.
     * @param[in] other 另一个CFList类实例。
     */
    CFList(const CFList<T>& other)
        : m_vector(other.m_vector) {
        
    }

    /**
     * 析构函数.
     * 在CFList实例被销毁时会被自动调用。
     * 在析构函数中进行内存释放等操作。
     */
    ~CFList() {

    }

    /**
     * 判断该列表是否为空。
     *
     * @return 返回该列表是否为空列表。
     * @return true:  列表为空。
     * @return false: 列表不为空。
     */
    bool isEmpty() const {
        return m_vector.empty();
    }

    /**
     * 将数据添加到列表的最后一位。
     *
     * @param[in] data 需要添加到列表中的数据。
     * @see insert()
     */
    void append(T data) {
        insert(data, -1);
    }

    /**
     * 将数据添加到列表中，指定的位置。
     *
     * @param[in] data 需要添加到列表中的数据。
     * @param[in] pos  目标位置， -1或大于列表长度，则添加到列表末尾。
     * @see append()
     */
    void insert(T data, CFInt pos) {
        CFInt vsize = m_vector.size();
        if (0 <= pos && vsize > pos) {
            m_vector.insert(m_vector.begin() + pos, data);
        } else {
            m_vector.push_back(data);
        }
    }

    /**
     * 将指定位置的数据从列表中移除。
     *
     * @param[in] pos 目标位置，若位置不在列表中，则不做处理。
     */
    void remove(CFInt pos) {
        CFInt vsize = m_vector.size();
        if (0 <= pos && vsize > pos) {
            m_vector.remove(m_vector.begin() + pos);
        } else {
            // Do nothing
        }
    }

    /**
     * 清除列表中储存的所有内容。
     *
     */
    void clear() {
        m_vector.clear();
    }

    /**
     * 获取列表中首位置的数据。
     *
     * @return 返回首位置的数据。
     * @see at()
     */
    T first() {
        return m_vector.front();
    }

    /**
     * 获取列表中末位置的数据。
     *
     * @return 返回末位置的数据。
     * @see at()
     */
    T last() {
        return m_vector.back();
    }

    /**
     * 获取列表中指定位置的数据。
     *
     * @return 返回指定位置的数据。
     */
    T at(CFInt pos) {
        CFInt vsize = m_vector.size();
        if (0 <= pos && vsize > pos) {
            return m_vector.at(pos);
        }
        return T();
    }

    /**
     * 获取列表中数据的个数。
     *
     * @return 返回当前列表中储存的数据个数。
     */
    CFInt size() {
        return m_vector.size();
    }

    /**
     * 获取列表中指定位置的数据。
     *
     * @return 返回指定位置的数据。
     * @see at()
     */
    T operator[] (CFInt pos) {
        return at(pos);
    }

    /**
     * 重载赋值运算符。
     *
     * @param[in] other 另一个CFList类实例。
     * @return 赋值后列表的引用。
     */
    CFList<T>& operator=(const CFList<T>& other) {
         this->m_vector = other.m_vector;
         return *this;
    }

private:
    std::vector<T> m_vector;
};

#endif // __CF_LIST_H__