#ifndef __CF_LOG_H__
#define __CF_LOG_H__

#include <Core/CFTypes.h>
#include <Core/CFDefines.h>
#include <stdarg.h>

#define CFLOGI(...) cf_log_Common(TAG, CFLOGLV_INFO, __VA_ARGS__);
#define CFLOGD(...) cf_log_Common(TAG, CFLOGLV_DEBUG, __VA_ARGS__);
#define CFLOGW(...) cf_log_Common(TAG, CFLOGLV_WARNING, __VA_ARGS__);
#define CFLOGE(...) cf_log_Common(TAG, CFLOGLV_ERROR, __VA_ARGS__);
#define CFLOGF(...) cf_log_Common(TAG, CFLOGLV_FATAL, __VA_ARGS__);

CF_EXPORT void cf_enable_debug();
CF_EXPORT void cf_disable_debug();
CF_EXPORT void cf_enable_info();
CF_EXPORT void cf_disable_info();

CF_EXPORT void cf_log_Common(const char* logtag, CFLogLevel level, const char* format, ...);

/**
 * \brief CFLog 类.
 *
 * 该类用于将日志信息打印到屏幕上或指定文件中。
 * 是libcdos-fw日志系统的重要组成部分。
 *
 */
class CFLog {
public:
    /**
     * 增加log输出的级别。
     *
     * 函数用于打开调试级别及调试以下级别的log信息。
     * 
     * 可用于增加单个log输出级别，也可以增加多个输出级别。
     * 
     * @param[in] mask 需要添加的日志级别掩码，可使用位或("|")操作实现增加多个输出级别。
     * @see CFLogMask
     */
    static void addLevelMask(unsigned int mask);

    /**
     * 关闭log输出的级别。
     *
     * 函数用于关闭调试级别及调试以下级别的log信息。
     * 
     * 可用于关闭单个log输出级别，也可以关闭多个输出级别。
     * 
     * @param[in] mask 需要关闭的日志级别掩码，可使用位或("|")操作实现关闭多个输出级别。
     * @see CFLogMask
     */
    static void removeLevelMask(unsigned int mask);

    /**
     * 输出CFLOGLV_INFO级别日志
     *
     * 使用统一的格式来输出INFO级别的日志信息。
     * 
     * 若未使用addLevelMask()开启INFO级别，则无任何输出信息。
     * 
     * @param[in] tag    输出信息的标签。
     * @param[in] format 输出信息的格式。
     * @param[in] ...    可变参数，对格式中的内容进行填充。
     * @see logCommon()
     */
    static void i(const char* tag, const char* format, ...);

    /**
     * 输出CFLOGLV_DEBUG级别日志
     *
     * 使用统一的格式来输出DEBUG级别的日志信息。
     * 
     * 若未使用addLevelMask()开启DEBUG级别，则无任何输出信息。
     * 
     * @param[in] tag    输出信息的标签。
     * @param[in] format 输出信息的格式。
     * @param[in] ...    可变参数，对格式中的内容进行填充。
     * @see logCommon()
     */
    static void d(const char* tag, const char* format, ...);

    /**
     * 输出CFLOGLV_WARNING级别日志
     *
     * 使用统一的格式来输出警告级别的日志信息。
     * 
     * @param[in] tag    输出信息的标签。
     * @param[in] format 输出信息的格式。
     * @param[in] ...    可变参数，对格式中的内容进行填充。
     * @see logCommon()
     */
    static void w(const char* tag, const char* format, ...);

    /**
     * 输出CFLOGLV_ERROR级别日志
     *
     * 使用统一的格式来输出错误级别的日志信息。
     * 
     * @param[in] tag    输出信息的标签。
     * @param[in] format 输出信息的格式。
     * @param[in] ...    可变参数，对格式中的内容进行填充。
     * @see logCommon()
     */
    static void e(const char* tag, const char* format, ...);

    /**
     * 输出CFLOGLV_FATAL级别日志
     *
     * 使用统一的格式来输出致命错误级别的日志信息。
     * 
     * @param[in] tag    输出信息的标签。
     * @param[in] format 输出信息的格式。
     * @param[in] ...    可变参数，对格式中的内容进行填充。
     * @see logCommon()
     */
    static void f(const char* tag, const char* format, ...);

    /**
     * 输出日志信息
     *
     * 使用统一的格式来输出指定级别的日志信息。
     * 
     * @param[in] level  输出日志的级别。
     * @param[in] tag    输出信息的标签。
     * @param[in] format 输出信息的格式。
     * @param[in] ap     用于对格式中的内容进行填充。
     */
    static void logCommon(CFLogLevel level, const char* tag, const char* format, va_list ap);

private:
    CFLog();
    CFLog(const CFLog& );

    static CFLog* getInst();

    void log(CFLogLevel level, const char* tag, const char* format, va_list ap);

    void addMask(unsigned int mask);
    void removeMask(unsigned int mask);

    char* currentTimeStr();

private:
    static CFLog* s_inst;
    unsigned int m_mask;
};

#endif // __CF_LOG_H__