#ifndef __CF_MAP_H__
#define __CF_MAP_H__

#include <Core/CFTypes.h>
#include <map>

/**
 * \brief CFMap 容器类.
 *
 * CFMap是一个关联性容器，以特定顺序来储存 key - value(即：键值-数据)组合元素
 * 在CFMap中，如果数据被储存在CFMap中，并与特定键值进行关联， 则该键值通常用于标识该数据与排序。
 * 
 *
 * 在内部，CFMap中的每个元素总是通过它的键值进行排序， 而键值是通过其内部一个特定的比较方式来确定顺序的。
 *
 * CFMap容器通过键值来访问个别的元素，通常会比未排序的Map容器慢，但是允许对子集按其顺序进行定向迭代。
 * 在CFMap中，对数据元素的键值使用“[]”运算符，即可对该数据元素进行定向访问。
 *
 */
template<typename K, typename V>
class CFMap {
public:
    /**
     * 默认构造函数.
     */
    CFMap() : m_map(std::map<K, V>()) {}

    /**
     * 拷贝构造函数.
     * @param[in] other 另一个CFMap类实例。
     */
    CFMap(const CFMap& other)
        : m_map(other.m_map) {

    }

    /**
     * 析构函数.
     * 在CFMap实例被销毁时会被自动调用。
     * 在析构函数中进行内存释放等操作。
     */
    ~CFMap() {}

    /**
     * 判断该map是否为空。
     *
     * @return 返回该map是否为空列表。
     * @return true:  map为空。
     * @return false: map不为空。
     */
    bool isEmpty() const {
        return m_map.empty();
    }

    /**
     * 判断该map是否包含key键。
     *
     * @param[in] key 需要判断的键值。
     * @return 返回该map是否包含key键。
     * @return true:  map中包含key键。
     * @return false: map中不包含key键。
     */
    bool contain(K key) const {
        return (m_map.end() != m_map.find(key));
    }

    /**
     * 将key-val进行关联并插入到map中。
     *
     * @param[in] key 需要插入的键值。
     * @param[in] val 需要与key关联的数据元素。
     */
    void insert(K key, V val) {
        m_map.insert( std::pair<K, V>(key, val) );
    }

    /**
     * 将key与其关联数据元素从map中删除。
     *
     * @param[in] key 需要移除的键值。
     */
    void remove(K key) {
        m_map.erase(key);
    }

    /**
     * 清除该CFMap中的所有元素。
     *
     */
    void clear() {
        m_map.clear();
    }

    /**
     * 从map中获取与key相关联的数据元素。
     *
     * @param[in] key 需要获取的数据元素的键值。
     * @return 返回与key相关联的数据元素。
     */
    V value(K key) {
        return m_map[key];
    }

    /**
     * 从map中获取与key相关联的数据元素。
     *
     * @param[in] key 需要获取的数据元素的键值。
     * @return 返回与key相关联的数据元素。
     * @see value()
     */
    V operator[] (K key) {
        return value(key);
    }

private:
    std::map<K, V> m_map;
};

#endif // __CF_MAP_H__