#ifndef __CF_MATH_H__
#define __CF_MATH_H__

/**
 * \brief CFMath 命名空间.
 *
 * CFMath命名空间，主要用于处理数学计算相关的函数与方法。
 * 
 * 提供了可用于三角函数计算的函数、常数等，如 sin(), cos()等；<br>
 * 提供了可用于浮点数运算的函数，如 floor(), round()等；<br>
 * 提供了幂运算或开方运算的函数，如 sqrt(), pow()等；<br>
 * 提供了对数运算，如 log(), log10()等；<br>
 * 提供了自然常数运算，如 exp(), exp2()等；<br>
 * 具体请参照CFMath文档。
 */
namespace CFMath {

    /**
     * 圆周率常数。
     */
    const double PI = 3.14159265358979323846;

    /**
     * 正弦函数。
     *
     * 通过弧度计算出其正弦值。
     * @param [in] x 弧度值， 1弧度等到180/PI。
     * @return 返回x弧度的正弦值。
     */
    double sin (double x);

    /**
     * 正弦函数.
     *
     * 通过弧度计算出其正弦值。
     * @param [in] x 弧度值， 1弧度等到180/PI。
     * @return 返回x弧度的正弦值。
     */
    float sin (float x);

    /**
     * 正弦函数.
     *
     * 通过弧度计算出其正弦值。
     * @param [in] x 弧度值， 1弧度等到180/PI。
     * @return 返回x弧度的正弦值。
     */
    long double sin (long double x);

    /**
     * 反正弦函数.
     *
     * 通过正弦值计算出其对应的弧度值。
     * @param [in] x 正弦值， 取值范围:[-1, +1]
     * @return 返回正弦值x所对应的弧度值，取值范围：[-PI/2, +PI/2]。
     */
    double asin (double x);

    /**
     * 反正弦函数.
     *
     * 通过正弦值计算出其对应的弧度值。
     * @param [in] x 正弦值， 取值范围:[-1, +1]
     * @return 返回正弦值x所对应的弧度值，取值范围：[-PI/2, +PI/2]。
     */
    float asin (float x);

    /**
     * 反正弦函数.
     *
     * 通过正弦值计算出其对应的弧度值。
     * @param [in] x 正弦值， 取值范围:[-1, +1]
     * @return 返回正弦值x所对应的弧度值，取值范围：[-PI/2, +PI/2]。
     */
    long double asin (long double x);

    /**
     * 余弦函数.
     *
     * 通过弧度计算出其余弦值。
     * @param [in] x 弧度值， 1弧度等到180/PI。
     * @return 返回x弧度的余弦值。
     */
    double cos (double x);

    /**
     * 余弦函数.
     *
     * 通过弧度计算出其余弦值。
     * @param [in] x 弧度值， 1弧度等到180/PI。
     * @return 返回x弧度的余弦值。
     */
    float cos (float x);

    /**
     * 余弦函数.
     *
     * 通过弧度计算出其余弦值。
     * @param [in] x 弧度值， 1弧度等到180/PI。
     * @return 返回x弧度的余弦值。
     */
    long double cos (long double x);

    /**
     * 反余弦函数.
     *
     * 通过余弦值计算出其对应的弧度值。
     * @param [in] x 余弦值， 取值范围:[-1, +1]
     * @return 返回余弦值x所对应的弧度值，取值范围：[0, PI]。
     */
    double acos (double x);

    /**
     * 反余弦函数.
     *
     * 通过余弦值计算出其对应的弧度值。
     * @param [in] x 余弦值， 取值范围:[-1, +1]
     * @return 返回余弦值x所对应的弧度值，取值范围：[0, PI]。
     */
    float acos (float x);

    /**
     * 反余弦函数.
     *
     * 通过余弦值计算出其对应的弧度值。
     * @param [in] x 余弦值， 取值范围:[-1, +1]
     * @return 返回余弦值x所对应的弧度值，取值范围：[0, PI]。
     */
    long double acos (long double x);

    /**
     * 正切函数.
     *
     * 通过弧度计算出其正切值。
     * @param [in] x 弧度值， 1弧度等到180/PI。
     * @return 返回x弧度的正切值。
     */
    double tan (double x);

    /**
     * 正切函数.
     *
     * 通过弧度计算出其正切值。
     * @param [in] x 弧度值， 1弧度等到180/PI。
     * @return 返回x弧度的正切值。
     */
    float tan (float x);

    /**
     * 正切函数.
     *
     * 通过弧度计算出其正切值。
     * @param [in] x 弧度值， 1弧度等到180/PI。
     * @return 返回x弧度的正切值。
     */
    long double tan (long double x);

    /**
     * 反正切函数.
     *
     * 通过正切值计算出其对应的弧度值。
     * @param [in] x 正切值。
     * @return 返回正切值x所对应的弧度值，取值范围：[-PI/2, +PI/2]。
     */
    double atan (double x);

    /**
     * 反正切函数.
     *
     * 通过正切值计算出其对应的弧度值。
     * @param [in] x 正切值。
     * @return 返回正切值x所对应的弧度值，取值范围：[-PI/2, +PI/2]。
     */
    float atan (float x);

    /**
     * 反正切函数.
     *
     * 通过正切值计算出其对应的弧度值。
     * @param [in] x 正切值。
     * @return 返回正切值x所对应的弧度值，取值范围：[-PI/2, +PI/2]。
     */
    long double atan (long double x);

    /**
     * 反正切函数.
     *
     * 该函数会返回y/x的反正切函数，返回值为弧度值。
     *
     * 若两个参数均为0，则会发生一个错误。
     *
     * @param [in] y 代表y坐标值。
     * @param [in] x 代表x坐标值。
     * @return 返回y/x的反正切函数，取值范围：[-PI, +PI]。
     */
    double atan2 (double y, double x);

    /**
     * 反正切函数.
     *
     * 该函数会返回y/x的反正切函数，返回值为弧度值。
     *
     * 若两个参数均为0，则会发生一个错误。
     *
     * @param [in] y 代表y坐标值。
     * @param [in] x 代表x坐标值。
     * @return 返回y/x的反正切函数，取值范围：[-PI, +PI]。
     */
    float atan2 (float y, float x);

    /**
     * 反正切函数.
     *
     * 该函数会返回y/x的反正切函数，返回值为弧度值。
     *
     * 若两个参数均为0，则会发生一个错误。
     *
     * @param [in] y 代表y坐标值。
     * @param [in] x 代表x坐标值。
     * @return 返回y/x的反正切函数，取值范围：[-PI, +PI]。
     */
    long double atan2 (long double y, long double x);

    /**
     * 幂运算函数。
     *
     * 该函数会返回base的exponent次幂运算。
     *
     *
     * @param [in] base 幂运算的底数。
     * @param [in] exponent 幂运算的指数。
     * @return 返回以base为底，exponent次幂的结果。
     */
    double pow (double base, double exponent);

    /**
     * 幂运算函数。
     *
     * 该函数会返回base的exponent次幂运算。
     *
     *
     * @param [in] base 幂运算的底数。
     * @param [in] exponent 幂运算的指数。
     * @return 返回以base为底，exponent次幂的结果。
     */
    float pow (float base, float exponent);

    /**
     * 幂运算函数。
     *
     * 该函数会返回base的exponent次幂运算。
     *
     *
     * @param [in] base 幂运算的底数。
     * @param [in] exponent 幂运算的指数。
     * @return 返回以base为底，exponent次幂的结果。
     */
    long double pow (long double base, long double exponent);

    /**
     * 幂运算函数。
     *
     * 该函数会返回base的exponent次幂运算。
     *
     *
     * @param [in] base 幂运算的底数。
     * @param [in] exponent 幂运算的指数。
     * @return 返回以base为底，exponent次幂的结果。
     */
    double pow (double base, int exponent);

    /**
     * 幂运算函数。
     *
     * 该函数会返回base的exponent次幂运算。
     *
     *
     * @param [in] base 幂运算的底数。
     * @param [in] exponent 幂运算的指数。
     * @return 返回以base为底，exponent次幂的结果。
     */
    long double pow (long double base, int exponent);

    /**
     * 平方根函数。
     *
     * 该函数会返回x平方根。
     *
     *
     * @param [in] x 需要求平方根的数值。
     * @return 返回x的平方根。
     */
    double sqrt (double x);

    /**
     * 平方根函数。
     *
     * 该函数会返回x平方根。
     *
     *
     * @param [in] x 需要求平方根的数值。
     * @return 返回x的平方根。
     */
    float sqrt (float x);

    /**
     * 平方根函数。
     *
     * 该函数会返回x平方根。
     *
     *
     * @param [in] x 需要求平方根的数值。
     * @return 返回x的平方根。
     */
    long double sqrt (long double x);

    /**
     * 立方根函数。
     *
     * 该函数会返回x的立方根。
     *
     *
     * @param [in] x 需要求立方根的数值。
     * @return 返回x的立方根。
     */
    double cbrt (double x);

    /**
     * 立方根函数。
     *
     * 该函数会返回x的立方根。
     *
     *
     * @param [in] x 需要求立方根的数值。
     * @return 返回x的立方根。
     */
    float cbrt (float x);

    /**
     * 立方根函数。
     *
     * 该函数会返回x的立方根。
     *
     *
     * @param [in] x 需要求立方根的数值。
     * @return 返回x的立方根。
     */
    long double cbrt (long double x);

    /**
     * 向上取整函数。
     *
     * 该函数会返回不小于x的最小整数值。
     *
     * @param [in] x 需要取整的数值。
     * @return 返回不小于x的最小整数值。
     */
    double ceil (double x);

    /**
     * 向上取整函数。
     *
     * 该函数会返回不小于x的最小整数值。
     *
     * @param [in] x 需要取整的数值。
     * @return 返回不小于x的最小整数值。
     */
    float ceil (float x);

    /**
     * 向上取整函数。
     *
     * 该函数会返回不小于x的最小整数值。
     *
     * @param [in] x 需要取整的数值。
     * @return 返回不小于x的最小整数值。
     */
    long double ceil (long double x);

    /**
     * 向下取整函数。
     *
     * 该函数会返回不大于x的最大整数值。
     *
     * @param [in] x 需要取整的数值。
     * @return 返回不大于x的最大整数值。
     */
    double floor (double x);

    /**
     * 向下取整函数。
     *
     * 该函数会返回不大于x的最大整数值。
     *
     * @param [in] x 需要取整的数值。
     * @return 返回不大于x的最大整数值。
     */
    float floor (float x);

    /**
     * 向下取整函数。
     *
     * 该函数会返回不大于x的最大整数值。
     *
     * @param [in] x 需要取整的数值。
     * @return 返回不大于x的最大整数值。
     */
    long double floor (long double x);

    /**
     * 四舍五入取整函数。
     *
     * 该函数会返回x四舍五入后的整数值。
     *
     * @param [in] x 需要取整的数值。
     * @return 返回x四舍五入后的整数值。
     */
    double round (double x);

    /**
     * 四舍五入取整函数。
     *
     * 该函数会返回x四舍五入后的整数值。
     *
     * @param [in] x 需要取整的数值。
     * @return 返回x四舍五入后的整数值。
     */
    float round (float x);

    /**
     * 四舍五入取整函数。
     *
     * 该函数会返回x四舍五入后的整数值。
     *
     * @param [in] x 需要取整的数值。
     * @return 返回x四舍五入后的整数值。
     */
    long double round (long double x);

    /**
     * 绝对值函数。
     *
     * 该函数会返回x的绝对值。
     *
     * @param [in] x 需要取绝对值的数值。
     * @return 返回x的绝对值。
     */
    double abs (double x);

    /**
     * 绝对值函数。
     *
     * 该函数会返回x的绝对值。
     *
     * @param [in] x 需要取绝对值的数值。
     * @return 返回x的绝对值。
     */
    float abs (float x);

    /**
     * 绝对值函数。
     *
     * 该函数会返回x的绝对值。
     *
     * @param [in] x 需要取绝对值的数值。
     * @return 返回x的绝对值。
     */
    long double abs (long double x);
}

#endif // __CF_MATH_H__