#ifndef __CF_OBJECT_H__
#define __CF_OBJECT_H__

#include <Core/CFDefines.h>
#include <glibmm.h>

CF_NAMESPACE_BEGIN

/**
 * 支持库的基础类。
 *
 * 支持库最基础的类， 需要使用支持库特性的类，都需要继承自本类。<br>
 * 本类为抽象类，无法被实例化，只可以被继承。
 * 
 * CFObject中，包含了支持库与实际需要使用的实例的映射机制，若需要使用该机制，则需要注意以下几个函数的使用：<br>
 * setRealObject() 函数， 可以将继承类与实际使用类的实例进行关联，并将其储存在一个全局的映射表中; 需要获取<br>
 * 实际使用的实例，可以使用 getObj() 函数来获取该实例的指针; 若需要通过实际使用的类实例来获取与其绑定的支持<br>
 * 库类实例， 则需要使用 getCFObject() 来获取。
 *
 * 若需要在类实例绑定后， 对使用类进行信号绑定或其它必要操作时，需要重写父类的 onRealObjectInit() 函数， <br>
 * 在子类继承并重写父类的 onRealObjectInit() 函数时，请在其内部调用父类的 onRealObjectInit() 函数，以便<br>
 * 父类进行必要的操作。
 */
class CFObject : public Glib::Object {
public:

    /**
     * 析构函数。
     *
     * 在类被销毁的时候会被自动调用。
     */
    virtual ~CFObject();

    /**
     * 获取支持库类实例所绑定的对象实例。
     *
     * 获取通过 setRealObject() 函数所绑定的对象实例。
     * @return 返回所绑定的对象实例。
     */
    void* getObj();

    /**
     * 通过对象实例获取所对应的 CFObject的子类对象实例。
     *
     * 使用 setRealObject() 函数将支持库类实例与所需要使用的对象实例进行绑定。<br>
     * 可以使用该函数通过使用的对象实例来获取与其绑定的支持库类实例。
     *
     * @param[in] realObj 对象实例。
     * @return 返回与realObj绑定的支持库类对象实例。
     */
    static CFObject* getCFObject(void* realObj);

protected:
    /**
     * 构造函数
     *
     * CFObject类的默认构造函数。<br>
     * 本构造函数为保护的，无法被直接实例化，但允许被继承。
     * 
     */
    CFObject();

    /**
     * 将自身实例与需要使用的对象实例进行绑定。
     *
     * 将自身实例与参数 obj 进行绑定， 并储存在一个全局的映射表中，<br>
     * 以方便使用 getObj() 或 getCFObject() 进行获取。
     * 
     * @param[in] obj 需要绑定的对象实例。
     */
    void setRealObject(void* obj);

    /**
     * 对象绑定后进行初始化的回调函数。
     *
     * 在使用 setRealObject() 绑定对象实例后， 本回调函数即会被调用，<br>
     * 若需要在类实例绑定后， 对使用类进行信号绑定或其它必要操作时，需要重写父类的 onRealObjectInit() 函数， <br>
     * 在子类继承并重写父类的 onRealObjectInit() 函数时，请在其内部调用父类的 onRealObjectInit() 函数，以便<br>
     * 父类进行必要的操作。
     * 
     */
    virtual void onRealObjectInit();

    /**
     * 对象解除绑定前的回调函数。
     *
     * 若需要在类实例解除绑定前， 对使用类进行信号解除绑定或其它必要操作时，需要重写父类的 onRealObjectFinalize() 函数， <br>
     * 在子类继承并重写父类的 onRealObjectFinalize() 函数时，请在该函数返回前调用父类的 onRealObjectFinalize() 函数，以便<br>
     * 父类进行必要的操作。
     * 
     */
    virtual void onRealObjectFinalize();

private:
    void* m_realObj;

    static std::map<void*, CFObject*> s_objMap;
};

CF_NAMESPACE_END

#endif // __CF_OBJECT_H__