#ifndef __CFSTRING_H__
#define __CFSTRING_H__

#include <Core/CFTypes.h>
#include <string>

class CFStringList;

/**
 * \brief CFString 字符串类.
 *
 * CFString是字符串类，用于处理各种字符串相关操作；
 * 其中包括但不限于：
 *    字符串读取
 *    字符串连接
 *    字符串截取
 *    字符串分割
 *    字符串查找
 *    字符串比较
 */
class CFString {

public:
    /**
     * 默认构造函数.
     */ 
    CFString();

    /**
     * 构造函数.
     * @param[in] str 以'\0'结尾的C语言格式字符串，可以包含中文。
     * @see CFString(const CFUChar* str);
     */
    CFString(const CFChar* str);

    /**
     * 构造函数.
     * @param[in] str 以'\0'结尾的C语言格式字符串，可以包含中文。
     * @see CFString(const CFChar* str);
     */
    CFString(const CFUChar* str);

    /**
     * 拷贝构造函数.
     * @param[in] other 另一个CFString类实例。
     */
    CFString(const CFString& other);

    /**
     * 析构函数.
     * 在字符串被销毁时会被自动调用。
     * 在析构函数中进行内存释放等操作。
     */
    ~CFString();

    /**
     * 获取C字符串长度.
     * 获取字符串的字节长度，不考虑编码。
     * @return 字符串的字节长度。
     * @see length();
     */
    CFUInt clength();

    /**
     * 获取字符串长度.
     * 获取字符串的字长度，考虑编码(目前只支持ASCII与UTF-8编码)，因此中文中一个汉字长度为1。
     * @return 字符串中字符的个数。
     * @see clength();
     */
    CFUInt length();

    /**
     * 获取C语言字符串.
     * 获取C语言字符串。
     * @return C语言字符串。
     * @see toStdString();
     * @see toUtf8();
     */
    char* toCString();

    /**
     * 获取C++标准库字符串.
     * 获取C++标准库字符串。
     * @return C++标准库字符串。
     * @see toCString();
     * @see toUtf8();
     */
    std::string toStdString();

    /**
     * 获取UTF-8编码字符串.
     * 获取UTF-8编码字符串。
     * @return UTF-8编码字符串。
     * @see toCString();
     * @see toStdString();
     */
    CFUChar* toUtf8();

    /**
     * 字符串分割.
     * 使用参数splitStr来对字符串进行分割，并生成一个字符串数组返回给调用者。
     * @param[in] splitStr 用以分割字符串的分割符，可以是包含多个字符的字符串，也可以是只有单个字符的字符串，可以包含UTF-8字符。
     * @return 通过splitStr分割后生成的字符串列表。
     */
    CFStringList split(const CFString& splitStr);

    /**
     * 字符串截取.
     * 从 pos 位置开始截取 len 个字符，并将该字符串返回给调用者。
     * @param[in] pos 需要截取字符串的起始位置，范围：0～length(), 一个UTF-8字符算一个字符。
     * @param[in] len 需要截取字符串的长度，一个UTF-8字符算一个字符，若需要截取 pos 后的全部字符串，则传入 -1 .
     * @return 截取操作后的字符串。
     */
    CFString sub(CFUInt pos, CFInt len);

    /**
     * 字符串查找.
     * 从 from 位置开始查找 chr 字符，caseSensitive参数表示是否区分大小写。
     * @param[in] chr 需要查找的字符。
     * @param[in] from 起始位置，默认为0。
     * @param[in] caseSensitive 是否区分大小写，区分大小写为true，否则为false，默认区分大小写.
     * @return 返回chr字符第一次找到的位置，返回-1表示未找到chr。
     */
    CFInt indexOf(CFUChar chr, int from = 0, bool caseSensitive = true);

    /**
     * 字符串查找.
     * 从 from 位置开始查找 str 字符串，caseSensitive参数表示是否区分大小写。
     * @param[in] str 需要查找的字符串(可以包含UTF-8字符)。
     * @param[in] from 起始位置，默认为0。
     * @param[in] caseSensitive 是否区分大小写，区分大小写为true，否则为false，默认区分大小写.
     * @return 返回str字符串第一次找到的位置，返回-1表示未找到str字符串。
     */
    CFInt indexOf(const CFString& str, int from = 0, bool caseSensitive = true);

    /**
     * 判断是否包含UTF-8编码的字符.
     *
     * @return 是否包含UTF-8编码的字符，true为包含UTF-8字符，否则为不包含。
     */
    bool isUtf8() const ;

    /**
     * 判断是否为空字符串.
     *
     * @return 是否为空字符串，true为空字符串，否则为非空字符串。
     */
    bool isEmpty() const;

    /**
     * 判断字符串是否与 other 字符串相同.
     *
     * @param[in] other 需要与之比较的字符串。
     * @return 是否与 other 字符串相同，true为相同，否则为不同。
     * @see compare()
     */
    bool equal(const CFString& other);

    /**
     * 比较字符串和 other 字符串.
     *
     * @param[in] other 需要与之比较的字符串。
     * @return 0 为与other字符串相同。
     * @return >0 则字符串大于 other 字符串。
     * @return <0 则字符串小于 other 字符串。
     * @see equal()
     */
    CFInt compare(const CFString& other);

    void display();

public:
    friend CFString operator+(const CFString& str1, const CFString& str2);
    friend CFString operator+(const CFString& str1, const CFUChar* str2);
    friend CFString operator+(const CFUChar* str1, const CFString& str2);
    friend CFString operator+(const CFString& str1, const CFChar* str2);
    friend CFString operator+(const CFChar* str1, const CFString& str2);
    // friend CFString operator+(const CFUChar* str1, const CFUChar* str2);

    CFString& operator=(const CFString& str1);
    CFString& operator+=(const CFString& str1);

    bool operator==(const CFString& other);
    bool operator!=(const CFString& other);
    bool operator>(const CFString& other);
    bool operator>=(const CFString& other);
    bool operator<(const CFString& other);
    bool operator<=(const CFString& other);

private:
    CFUInt getUTF8length();
    void create(const CFUChar* strdata);

    // 将字符所在位置的值转换成字符数据中的索引值.
    // 若为纯ascii字符串，则字符所在位置与其索引值相同.
    // 若为含有UTF-8字符的字符串，则字符所在位置比其索引值小.
    CFUInt convertPosToIndex(CFUInt pos);

    // 将字符数据中的索引值转换成字符所在位置的值
    // 若为纯ascii字符串，则字符所在位置与其索引值相同.
    // 若为含有UTF-8字符的字符串，则字符所在位置比其索引值小.
    CFInt convertIndexToPos(CFInt index);

    CFUInt dataLength(const CFUChar* strdata);
    CFUInt wordWidth(CFUChar chr) const;
    bool charEqual(CFUChar destChr, CFUChar srcChr, bool caseSensitive = true);

private:
    CFUChar* m_data;
    CFUInt m_strLen;
    CFUInt m_utfLen;
};

#endif // __CFSTRING_H__
