﻿/** \file
 *  结构与枚举类型定义.
 */
#ifndef __CF_TYPES_H__
#define __CF_TYPES_H__

#include <stdint.h>

/** \addtogroup 结构与枚举类型定义
   @{ */

/**
 *  日志级别掩码过滤参数
 *  
 *  用于将掩码无效的位，通过位运算过滤掉。以避免不必要的干扰。
 */
#define CFLOG_MASK_GATE (0x1F)

/**
 * \enum CFLogLevel
 * \brief 日志级别枚举类型
 *
 * 用于规定使用者打印日志的优先级别。
 * 按照统一的格式，规范日志输出。
 */
typedef enum {
    /**
     * 最低优先级别。
     *
     * 主要用于打印调试日志
     * 默认关闭，可以设置开启
     * @see CFLog::addLevelMask()
     * @see CFLOGMSK_INFO
     */
    CFLOGLV_INFO = 0,

    /**
     * 调试级别日志。
     *
     * 主要用于打印调试日志
     * 默认关闭，可以设置开启
     * @see CFLog::addLevelMask()
     * @see CFLOGMSK_DEBUG
     */
    CFLOGLV_DEBUG,

    /**
     * 警告级别日志。
     *
     * 主要用于打印警告信息
     * 默认开启，无法关闭。
     */
    CFLOGLV_WARNING,

    /**
     * 错误级别日志。
     *
     * 主要用于打印错误信息或可能引起系统异常的错误信息。
     *
     * 默认开启，无法关闭。
     */
    CFLOGLV_ERROR,

    /**
     * 最高优先级别日志。
     *
     * 主要用于打印致命错误信息或会引起系统异常的错误信息。
     *
     * 使用该级别打印日志会引起主动crash，因此非致命错误，请勿使用。
     *
     * 默认开启，无法关闭。
     */
    CFLOGLV_FATAL
} CFLogLevel;

/**
 * \enum CFLogMask
 * \brief 日志级别掩码枚举类型
 *
 * 用于开启或关闭一个或多个日志级别的输出。
 */
typedef enum {
    /**
     * INFO级别掩码
     *
     * 用于开启或关闭CFLOGLV_INFO级别日志输出。
     * 
     * @see CFLog::i()
     * @see CFLOGLV_INFO
     */
    CFLOGMSK_INFO = (1 << 0),

    /**
     * DEBUG级别掩码
     *
     * 用于开启或关闭CFLOGLV_DEBUG级别日志输出。
     * 
     * @see CFLog::d()
     * @see CFLOGLV_DEBUG
     */
    CFLOGMSK_DEBUG = (1 << 1),

    /**
     * WARNING级别掩码
     *
     * 目前警告级别的日志无法被关闭，因此本项仅做声明。
     * 
     * @see CFLog::w()
     * @see CFLOGLV_WARNING
     */
    CFLOGMSK_WARNING = (1 << 2),

    /**
     * ERROR级别掩码
     *
     * 目前错误级别的日志无法被关闭，因此本项仅做声明。
     * 
     * @see CFLog::e()
     * @see CFLOGLV_ERROR
     */
    CFLOGMSK_ERROR = (1 << 3),

    /**
     * FATAL级别掩码
     *
     * 目前致命错误级别的日志无法被关闭，因此本项仅做声明。
     * 
     * @see CFLog::f()
     * @see CFLOGLV_FATAL
     */
    CFLOGMSK_FATAL = (1 << 4)
} CFLogMask;

/**
 * \enum CFWeek
 * 星期的枚举值， 从周日开始计数(0)
 */
typedef enum {
    CFWEEK_SUNDAY = 0,  ///< 周日
    CFWEEK_MONDAY,  ///< 周一
    CFWEEK_TUESDAY,  ///< 周二
    CFWEEK_WEDNESDAY,  ///< 周三
    CFWEEK_THURSDAY,  ///< 周四
    CFWEEK_FRIDAY,  ///< 周五
    CFWEEK_SATURDAY  ///< 周六
} CFWeek;

typedef void* CFPointer;
typedef unsigned int CFSizeType;
typedef unsigned int CFUInt;
typedef unsigned char CFUChar;
typedef int CFInt;
typedef char CFChar;
typedef short CFShort;
typedef unsigned short CFUShort;
typedef double CFDouble;
typedef bool CFBoolean;
typedef long CFLong;
typedef unsigned long CFULong;
typedef float CFFloat;


typedef signed char CFInt8;
typedef unsigned char CFUInt8;
typedef signed short CFInt16;
typedef unsigned short CFUInt16;
typedef signed int CFInt32;
typedef unsigned int CFUInt32;
typedef unsigned long CFSize;
typedef signed long CFInt64;
typedef unsigned long CFUInt64;
typedef signed long CFSSize;

/** @} */ /* 结构与枚举类型定义 */

#endif // __CF_TYPES_H__
