#ifndef __CF_BOX_H__
#define __CF_BOX_H__

#include <Gui/CFContainer.h>
#include <Gui/CFEnums.h>

CF_NAMESPACE_BEGIN

/**
 * \brief CFBox 该控件会以矩形区域来组织其所容纳的控件。
 *
 * CFBox容纳的子控件会被组织成一行或一列，这取决到 orientation 参数。<br>
 * 因此当CFBox被组织成一列时，所有CFBox容纳的子控件会被分配同样的宽度;<br>
 * 当CFBox被组织成一行时，所有CFBox容纳的子控件会被分配同样的高度。<br>
 *
 * CFBox使用一种打包的概念。
 * 打包的实现是将控件的实例添加到CFContainer的特定位置。<br>
 * CFBox支持两种指定的位置类型：
 * 从CFBox的起始位置开始打包和从CFBox的结束位置开始打包。<br>
 * 如：纵向的CFBox，起始位置为最上端，而结束位置为最下端;
 * 横向的CFBox，起始位置为最左端，<br>
 * 而结束位置为最右端。重复调用 pack_start()
 * 函数可以从起始位置到结束位置打包控件。<br>
 * 重复调用 pack_end()
 * 函数可以从结束位置到起始位置打包控件。也可以混合使用这两个函数进行布局。<br>
 * 使用 pack_start() 添加的最后一个控件会正好在使用 pack_end()
 * 添加的最后一个控件的前面。<br>
 *
 * 由于CFBox也是一个CFContainer类， 也可以使用 CFContainer::add()
 * 来添加控件，它的行会与使用<br>
 * pack_start() 一样。 也可以使用 CFContainer::remove() 来移除控件。
 *
 */
class CFBox : public CFContainer {
public:
    /**
     * 构造函数.
     *
     * 构造一个新的CFBox;
     *
     * @param[in] orientation CFBox的方向。
     * @param[in] spacing 所容纳控件间的距离，以像素为单位。
     */
    CFBox(CFOrientation orientation = CFORIENTATION_HORIZONTAL,
          int spacing = 0);

    /**
     * 析构函数.
     */
    virtual ~CFBox();

    /**
     * 将控件插入到CFBox中。
     *
     * 方向为从起始位置到结束位置。<br>
     *
     * pack_start() 或 pack_end() 的 expand 参数， 指定了 child 控件<br>
     * 是否会随着 CFBox 的拉伸而进行缩放。
     *
     * pack_start() 或 pack_end() 的 fill 参数, 指定了child控件是否<br>
     * 填充 CFBox 的剩余空间。
     *
     * spacing(构造时的参数)和 padding
     * 参数的区别是：spacing会被加到两个对象之间，<br>
     * 而 padding 会被加到对象的两边。
     *
     * @param[in] child 需要加入到 CFBox 的控件对象。
     * @param[in] expand true: child 会随着CFBox拉伸而缩放，<br> false:
     * child不会随着CFBox拉伸而缩放。
     * @param[in] fill 若expand为false时，则该参数无效; true:
     * child会填充CFBox的剩余空间，<br> false: child不会填充CFBox的剩余空间。
     * @param[in] padding 在child对象两边附加的以像素为单位的空间。
     */
    void packStart(CFWidget& child, bool expand, bool fill, CFUInt padding = 0);

    /**
     * 将控件插入到CFBox中。
     *
     * 方向为从结束位置到起始位置。<br>
     *
     * @param[in] child 需要加入到 CFBox 的控件对象。
     * @param[in] expand true: child 会随着CFBox拉伸而缩放，<br> false:
     * child不会随着CFBox拉伸而缩放。
     * @param[in] fill 若expand为false时，则该参数无效; true:
     * child会填充CFBox的剩余空间，<br> false: child不会填充CFBox的剩余空间。
     * @param[in] padding 在child对象两边附加的以像素为单位的空间。
     */
    void packEnd(CFWidget& child, bool expand, bool fill, CFUInt padding = 0);
};

CF_NAMESPACE_END

#endif  // __CF_BOX_H__