// Copyright 2017, CDOS.
// All rights reserved.
//
// Author: CDOS Support Team <support_os@sari.ac.cn>
//
// A widget used to choose from a list of items.
#ifndef __CF_COMBOBOX_H__
#define __CF_COMBOBOX_H__

#include "Gui/CFBin.h"
#include "Gui/CFEntry.h"
#include "Gui/CFEnums.h"
#include <gdkmm.h>

CF_NAMESPACE_BEGIN

class CFComboBox : public CFBin {
public:
    /** Creates a new empty ComboBox, optionally with an entry.
     * @param has_entry If this is true then this will have an Entry widget.
     */
    explicit CFComboBox(bool has_entry = false);

    ~CFComboBox();

    /** Sets the wrap width of @a combo_box to be @a width. The wrap width is basically
     * the preferred number of columns when you want the popup to be layed out
     * in a table.
     *
     * @param width Preferred number of columns.
     */
    void setWrapWidth(int width);

    /** Returns the wrap width which is used to determine the number of columns
     * for the popup menu. If the wrap width is larger than 1, the combo box
     * is in table mode.
     *
     * @return The wrap width.
     */
    int getWrapWidth();


    /** Sets the column with row span information for @a combo_box to be @a row_span.
     * The row span column contains integers which indicate how many rows
     * an item should span.
     *
     * @param row_span A column in the model passed during construction.
     */
    void setRowSpanColumn(int row_span);

    /** Returns the column with row span information for @a combo_box.
     *
     * @return The row span column.
     */
    int getRowSpanColumn();


    /** Sets the column with column span information for @a combo_box to be
     *  @a column_span. The column span column contains integers which indicate
     * how many columns an item should span.
     *
     * @param column_span A column in the model passed during construction.
     */
    void setColumnSpanColumn(int column_span);

    /** Returns the column with column span information for @a combo_box.
     *
     * @return The column span column.
     */
    int getColumnSpanColumn();

    /** Sets whether the popup menu should have a tearoff
     * menu item.
     *
     * @deprecated 没有替代的函数.
     *
     * @param add_tearoffs <tt>true</tt> to add tearoff menu items.
     */
    void setAddTearoffs(bool add_tearoffs = true);

    /** Gets the current title of the menu in tearoff mode. See
     * setAddTearoffs().
     *
     * @deprecated 没有替代的函数.
     *
     * @return The menu’s title in tearoff mode. This is an internal copy of the
     * string which must not be freed.
     */
    bool getAddTearoffs();


    /** Gets the current title of the menu in tearoff mode.
     *
     * @see setAddTearoffs().
     *
     * @deprecated 没有替代的函数.
     *
     * @return The menu’s title in tearoff mode. This is an internal copy of the
     * string which must not be freed.
     */

    Glib::ustring getTitle();

    /** Sets the menu’s title in tearoff mode.
     *
     * @deprecated 没有替代的函数.
     *
     * @param title A title for the menu in tearoff mode.
     */
    void setTitle(const Glib::ustring& title);

    /** Returns whether the combo box grabs focus when it is clicked
     * with the mouse. 
     *@see setFocusOnClick().
     *
     * @deprecated 使用 CFWidget::getFocusOnClick() 代替
     *
     * @return <tt>true</tt> if the combo box grabs focus when it is
     * clicked with the mouse.
     */
    bool getFocusOnClick();

    /** Sets whether the combo box will grab focus when it is clicked with
     * the mouse. Making mouse clicks not grab focus is useful in places
     * like toolbars where you don’t want the keyboard focus removed from
     * the main area of the application.
     *
     * @deprecated 使用 CFWidget::setFocusOnClick() 代替
     *
     * @param focus_on_click Whether the combo box grabs focus when clicked
     * with the mouse.
     */
    void setFocusOnClick(bool focus_on_click = true);

    /* get/set active item */

    /** Returns the index of the currently active item, or -1 if there’s no
     * active item. If the model is a non-flat treemodel, and the active item
     * is not an immediate child of the root of the tree, where
     * `path` is the CFTreePath of the active item.
     *
     * @return An integer which is the index of the currently active item,
     * or -1 if there’s no active item.
     */
    int getActiveRowNumber();

    /** Sets the active item of @a combo_box to be the item at @a index.
     *
     * @param index An index in the model passed during construction, or -1 to have
     * no active item.
     */
    void setActive(int index);

    /** Causes no item to be active. 
     * @see setActive().
     */
    void unsetActive();

    /** Causes no separators to be drawn.
     */
    void unsetRowSeparatorFunc();


    /** Sets whether the dropdown button of the combo box should be
     * always sensitive (CFSENSITIVITY_ON), never sensitive (CFSENSITIVITY_OFF)
     * or only if there is at least one item to display (CFSENSITIVITY_AUTO).
     *
     * @param sensitivity Specify the sensitivity of the dropdown button.
     */
    void setButtonSensitivity(CFSensitivityType sensitivity);

    /** Returns whether the combo box sets the dropdown button
     * sensitive or not when there are no items in the model.
     *
     * @return CFSENSITIVITY_ON if the dropdown button
     * is sensitive when the model is empty, CFSENSITIVITY_OFF
     * if the button is always insensitive or
     * CFSENSITIVITY_AUTO if it is only sensitive as long as
     * the model has one item to be selected.
     */
    CFSensitivityType getButtonSensitivity();


    /** Returns whether the combo box has an entry.
     *
     * @return Whether there is an entry in @a combo_box.
     */
    bool getHasEntry();

    /** Sets the model column which @a combo_box should use to get strings from
     * to be @a text_column. The column @a text_column in the model of @a combo_box
     * must be of type TYPE_STRING.
     *
     * This is only relevant if @a combo_box has been created with
     * CFComboBox::property_has_entry() as <tt>true</tt>.
     *
     * @param text_column A column in @a model to get the strings from for
     * the internal entry.
     */
    void setEntryTextColumn(int text_column);


    /** Returns the column which @a combo_box is using to get the strings
     * from to display in the internal entry.
     *
     * @return A column in the data source model of @a combo_box.
     */
    int getEntryTextColumn();


    /** Specifies whether the popup’s width should be a fixed width
     * matching the allocated width of the combo box.
     *
     * @param fixed Whether to use a fixed popup width.
     */
    void setPopupFixedWidth(bool fixed = true);

    /** Gets whether the popup uses a fixed width matching
     * the allocated width of the combo box.
     *
     * @return <tt>true</tt> if the popup uses a fixed width.
     */
    bool getPopupFixedWidth();


    /** Pops up the menu or dropdown list of @a combo_box.
     *
     * This function is mostly intended for use by accessibility technologies;
     * applications should have little use for it.
     */
    void popup();

    /** Pops up the menu or dropdown list of @a combo_box, the popup window
     * will be grabbed so only @a device and its associated pointer/keyboard
     * are the only Gdk::Devices able to send events to it.
     *
     * @param device A Gdk::Device.
     */
    void popup(const Glib::RefPtr<Gdk::Device>& device);

    /** Hides the menu or dropdown list of @a combo_box.
     *
     * This function is mostly intended for use by accessibility technologies;
     * applications should have little use for it.
     *
     */
    void popdown();

    /** Returns the column which @a combo_box is using to get string IDs
     * for values from.
     *
     * @return A column in the data source model of @a combo_box.
     */
    int getIdColumn();

    /** Sets the model column which @a combo_box should use to get string IDs
     * for values from. The column @a id_column in the model of @a combo_box
     * must be of type TYPE_STRING.
     *
     * @param id_column A column in @a model to get string IDs for values from.
     */
    void setIdColumn(int id_column);

    /** Returns the ID of the active row of @a combo_box.  This value is taken
     * from the active row and the column specified by the CFComboBox::property_id_column()
     * property of @a combo_box (see set_id_column()).
     *
     * The returned value is an interned string which means that you can
     * compare the pointer by value to other interned strings and that you
     * must not free it.
     *
     * If the Cdos::CFComboBox::property_id_column() property of @a combo_box is not set, or if
     * no row is active, or if the active row has a <tt>nullptr</tt> ID value, then <tt>nullptr</tt>
     * is returned.
     *
     * @return The ID of the active row, or <tt>nullptr</tt>.
     */
    Glib::ustring getActiveId();

    /** Changes the active row of @a combo_box to the one that has an ID equal to
     *  @a active_id, or unsets the active row if @a active_id is <tt>nullptr</tt>.  Rows having
     * a <tt>nullptr</tt> ID string cannot be made active by this function.
     *
     * If the Cdos::CFComboBox::property_id_column() property of @a combo_box is unset or if no
     * row has the given ID then the function does nothing and returns <tt>false</tt>.
     *
     * @deprecated 使用 set_active_id(). 该方法命名不规范, id实际上的类型是字符串, 而不是数值
     *
     * @param active_id The ID of the row to select, or <tt>nullptr</tt>.
     * @return <tt>true</tt> if a row with a matching ID was found.  If a <tt>nullptr</tt>
     * @a active_id was given to unset the active row, the function
     * always returns <tt>true</tt>.
     */
    void getActiveId(const Glib::ustring& active_id);

    /** Changes the active row of the combo box to the one that has an ID equal to
     * @a active_id.
     *
     * If property_id_column() is unset or if no row has the given ID then the function
     * does nothing and returns false.
     *
     * @param active_id The ID of the row that should be set to active.
     * @returns true if a row with a matching ID was found.
     */
    bool setActiveId(const Glib::ustring& active_id);

    /** @see CFBin::get_child().
     */
    CFEntry* getEntry();
};

CF_NAMESPACE_END

#endif  // __CF_COMBOBOX_H__
