// Copyright 2017, CDOS.
// All rights reserved.
//
// Author: CDOS Support Team <support_os@sari.ac.cn>
//
// CFNotification is a notification item class.

/** \file
 *  通知控件
 */

#ifndef __CF_NOTIFICATION_H__
#define __CF_NOTIFICATION_H__

#include <Gui/CFButton.h>
#include <Gui/CFImage.h>
#include <Gui/CFLabel.h>
#include <Core/CFDefines.h>
#include <Gui/CFFixed.h>
#include <Gui/CFBox.h>

CF_NAMESPACE_BEGIN

/** \addtogroup 结构与枚举类型定义
   @{ */

/** \enum CFNotificationUrgency
 * 用以表示通知紧急程度的枚举
 *
 * 默认是kUrgencyNormal
 */
enum CFNotificationUrgency {
    kUrgencyLow = 0, /**< 紧急程度低 */
    kUrgencyNormal, /**< 紧急程度一般 */
    kUrgencyCritical /**< 紧急程度高 */
};

/** \enum CFNotificationDestroyedReason
 * 通知关闭的原因
 *
 * kTimeout需要主动调用设置函数，单位毫秒
 */
enum CFNotificationDestroyedReason {
    kNoReason = 0, /**< 未定义，对应js里的undefine */
    kTimeout, /**< 表示通知超过了设定的过期时间 */
    kClicked /**< 表示通知被点击 */
};

/** \struct CFNotificationParams
 * \todo 使用参数待完善
 * 通知设定的详细参数
 *
 * 包括内容，消息体，图标等等。
 */
struct CFNotificationParams {
    bool custom_content; /**< 自定义内容 */
    Glib::ustring body; /**< 消息体 */
    CFImage image; /**< 图标 */
    bool title_markup; /**< 标题是否允许标记 */
    bool banner_markup; /**< 消息体是可以标记 */
    bool body_markup; /**< C版本里与bannerMarkup相同 */
    bool cleanable; /**< 是否可清除,默认 %FALSE */
};

/** @} */

const Glib::ustring kDateFormat = "%l:%M %p";  // _("%l:%M %p")

/**
 * \brief CFNotification 控件会以矩形区域来组织其所容纳的控件。
 *
 * 控件包括图标，时间标签，标题标签，内容标签。通知控件的高度会自适应内容，但并不是无限扩展。
 * 参考ubuntu的设定，通知标题最多3行，通知内容最多10行。
 */
class CFNotification : public CFButton {
public:
    /**
     * 构造函数
     *
     * 创建一个 CFNotification 类型的实例
     *
     * @param [in] icon_name 图标名称
     * @param [in] title 通知标题
     * @param [in] content 通知内容
     */
    CFNotification(const Glib::ustring& icon_name,
                   const Glib::ustring& title,
                   const Glib::ustring& content);

    /**
     * 析构函数.
     *
     * 在 CFNotification 被销毁时会被自动调用。
     * 在析构函数中进行内存释放等操作。
     */
    virtual ~CFNotification();

    /**
     * 获取 CFNotification 的图标
     *
     * @return 返回 CFNotification 的图标
     */
    CFImage& getIcon();

    /**
     * 设置 CFNotification 的图标
     *
     * 设置通过名称设置图标，图标尺寸是 CFBuiltinIconSize 类型
     *
     * @param [in] lable 图标名称
     * @param [in] icon_size 图标尺寸
     */
    void setIcon(const Glib::ustring& label, CFBuiltinIconSize icon_size);

    /**
     * 获取 CFNotification 的标题
     *
     * @return 返回 CFNotification 的标题
     */
    Glib::ustring getTitle();

    /**
     * 设置 CFNotification 的标题
     *
     * @param [in] title 通知标题
     */
    void setTitle(const Glib::ustring& title);

    /**
     * 获取 CFNotification 的创建时间
     *
     * 返回类型是 Glib::DateTime 类型。
     *
     * @return 返回 CFNotification 的创建时间
     */
    Glib::DateTime getCreateDateTime();

    /**
     * 设置 CFNotification 的时间标签
     *
     * @param [in] time 设置时间标签
     */
    void setTime(const Glib::ustring& time);

    /**
     * 设置 CFNotification 的param_cn
     *
     * @param [in] ms_time 设置过期时间
     */
    void setTimeout(CFUInt ms_time);  // 单位毫秒

    /**
     * 设置 CFNotification 的销毁的原因
     *
     * @param [in] reason 销毁的原因
     */
    void setReason(CFNotificationDestroyedReason reason);

    /**
     * 设置 CFNotification 的显示时间标签
     *
     * @param [in] show_time 是否显示时间标签
     */
    void setShowTime(bool show_time);

    /**
     * 获取 CFNotification 的紧急程度
     *
     * @return 返回 CFNotification 的紧急程度
     */
    CFNotificationUrgency getUrgency();

    /**
     * 设置 CFNotification 的紧急程度
     *
     * @param [in] urgency 紧急程度
     */
    void setUrgency(CFNotificationUrgency urgency);

public:
    /**
     * 槽函数类型：<br>
     *    void on_my_reason(CFNotificationDestroyedReason reason);
     *
     * 接收通知销毁的原因
     */
    sigc::signal<void, CFNotificationDestroyedReason> signalReason();

protected:
    virtual void onShow();

    virtual bool onTimeout();

    virtual void onClicked();

private:
    CFImage m_icon;  // left
    CFBox m_contentBox;  // right
    CFLabel m_time;
    Glib::DateTime m_createTime;
    CFLabel m_title;
    CFLabel m_content;
    CFBox m_actor;  // main container

    bool m_isNeedShowTime;  // member
    CFNotificationUrgency m_urgency;

    // signal and handler
    CFNotificationDestroyedReason m_reason;
    sigc::signal<void, CFNotificationDestroyedReason> m_reasonSignal;
};

CF_NAMESPACE_END

#endif  // __CF_NOTIFICATION_H__

/* ----------- test case ----------- */
// void signal_test(Cdos::NotificationDestroyedReason reason) {
//     std::cout << reason << std::endl;
// }
// bt.signal_reason().connect(sigc::ptr_fun(signal_test));
// bt.set_timeout(4000);
