// Copyright 2017, CDOS.
// All rights reserved.
//
// Author: CDOS Support Team <support_os@sari.ac.cn>
//
// CFPlacesSidebar that displays frequently-used places in the file system.
/**
 * \file 展示文件系统中最常用地址的控件。
 */
#ifndef __CF_PLACESSIDEBAR_H__
#define __CF_PLACESSIDEBAR_H__

#include "Gui/CFScrolledView.h"
#include <giomm.h>
#include "Gui/CFMenu.h"
#include <gdkmm.h>
#include <iostream>

CF_NAMESPACE_BEGIN

/**
 * \brief
 * 该控件展示文件系统中最常用地址的列表：用户家目录，用户书签，磁盘卷，和驱动设备。
 * <br>此控件长使用在文件选择或者文件管理以及类似的程序里。
 *
 * 侧边栏控件展示驱动和卷，在用户选择它们的时候，最自动挂载或者移除。<br>
 * 为了使用侧边栏控件, 应用至少需要链接 CFPlaceSidebar::signal_open_location
 * 信号。
 * 该信号在用户选择侧边栏的一个地址打开时发送。应用也应该在改变当前视图时，
 * 调用CFPlaceSidebar::set_location()。
 */
class CFPlacesSidebarPriv;
class CFPlacesSidebar : public CFScrolledView {
public:
    /**
     * 默认构造函数.
     */
    CFPlacesSidebar();

    /**
     * 析构函数.
     *
     * 在 CFPlacesSidebar 被销毁时会被自动调用。
     * 在析构函数中进行内存释放等操作。
     */
    virtual ~CFPlacesSidebar();

    /**
     * 设置 CFPlacesSidebar 的位置打开方式
     *
     * @param [in] flags 位置打开方式
     */
    void setOpenFlags(CFPlacesOpenFlags flags = CFPLACES_OPEN_NORMAL);

    /**
     * 获取 CFPlacesSidebar 的设置位置打开方式
     *
     * @return 返回打开方式
     */
    CFPlacesOpenFlags getOpenFlags();

    /**
     * 获取 CFPlacesSidebar 的地址
     *
     * 设置在侧边栏周围的窗口小部件中显示的位置，例如，在文件管理器中的文件夹视图中。
     * 相反的，如果它在地点列表中显示，侧边栏将突出显示该位置；如果该位置不在列表中的
     * 位置之间，它将取消高亮显示。
     */
    void setLocation(Glib::RefPtr<Gio::File>& file);

    /**
     * 获取 CFPlacesSidebar 的当前选中地址
     *
     * @return 返回选中地址的 GFile, 或者在没有可见的选中时，返回 nullptr
     */
    Glib::RefPtr<Gio::File> getLocation();

    /**
     * 设置 CFPlacesSidebar 是否在侧边栏显示一个桌面文件夹
     *
     * @param [in] show_desktop 是否在侧边栏显示一个桌面文件夹
     */
    void setShowDesktop(bool show_desktop = true);

    /**
     * 获取 CFPlacesSidebar 的是否有桌面文件夹显示
     *
     * @return 返回 CFPlacesSidebar 的桌面文件夹的显示状态
     */
    bool getShowDesktop();

    /**
     * 添加某些对用户有用的文件夹到侧边栏
     *
     * @param [in] location 要添加的快捷方式
     */
    void addShortcut(const Glib::RefPtr<Gio::File>& location);

    /**
     * 从侧边栏中移除快捷方式
     *
     * @param [in] location 快捷方式的路径
     */
    void removeShortcut(const Glib::RefPtr<Gio::File>& location);

    /**
     * 获取 CFPlacesSidebar 的快捷方式列表
     *
     * @return 返回 CFPlacesSidebar 快捷方式的一个SList。如果这个地址不是一个
     * 快捷方式，就什么都不做。
     */
    std::vector<Glib::RefPtr<Gio::File>> listShortcuts();

    /**
     * 设置 CFPlacesSidebar 是否显示一个链接网络服务器，默认是关闭的。
     *
     * @deprecated 建议将此功能与新的“其他位置”项下的驱动器和网络位置分组。
     *
     * @param [in] show_connect_to_server 是否显示链接服务器的项
     */
    void setShowConnectToServer(bool show_connect_to_server = true);

    /**
     * 获取 CFPlacesSidebar 是否显示链接网络服务器
     *
     * @deprecated 建议将此功能与新的“其他位置”项下的驱动器和网络位置分组。
     *
     * @return 侧边栏显示链接服务器，返回true
     */
    bool getShowConnectToServer();

public:
    /**
     * 槽函数类型：<br>
     *    int on_my_drag_action_requested(
     *        const Glib::RefPtr<Gdk::DragContext>& context,
     *        const Glib::RefPtr<Gio::File>& dest_file,
     *        const std::vector<Glib::RefPtr<Gio::File> >& source_file_list)
     *
     * @param [in] context 带有拖拽信息的Gdk::DragContext
     * @param [in] dest_file 目标文件按
     * @param [in] source_file_list 被拖拽的源文件列表
     *
     * @return more_text
     */
    sigc::signal<int,
                 const Glib::RefPtr<Gdk::DragContext>&,
                 const Glib::RefPtr<Gio::File>&,
                 const std::vector<Glib::RefPtr<Gio::File>>&>
    signalDragActionRequested();

    /**
     * 槽函数类型：<br>
     *    void on_my_drag_perform_drop(
     *       const Glib::RefPtr<Gio::File>& dest_file,
     *       const std::vector< Glib::RefPtr<Gio::File> >& source_file_list,
     *       int action)
     *
     * 当用户完成拖放操作并且侧边栏的项目之一是目标时，位置侧边栏会发出此信号。
     * 这个项目在dest_file中，source_file_list有被删除的文件列表，应该被复制/移动等。
     * 基于指定的action。
     *
     * @param [in] dest_file 目标文件
     * @param [in] source_file_list 丢弃的文件列表
     * @param [in] action 指定的动作
     */
    sigc::signal<void,
                 const Glib::RefPtr<Gio::File>&,
                 const std::vector<Glib::RefPtr<Gio::File>>&,
                 int>
    signalDragPerformDrop();

    /**
     * 槽函数类型：<br>
     *    bool on_my_xx(CFPlacesSidebar param1);
     *
     * @param [in] param1 more_text
     *
     * @return more_text
     */
    sigc::signal<int, int> signalDragActionAsk();

    /**
     * 槽函数类型：<br>
     *    int on_my_drag_action_ask(int actions)
     *
     * 当需要请求应用程序弹出菜单以询问用户执行哪个拖动操作时，位置边栏会发出此信号。
     *
     * @param [in] actions 需要被询问的可能拖拽动作
     *
     * @return 侧边栏应该传递到拖放侧的最终拖放动作
     */
    sigc::signal<void, const Glib::RefPtr<Gio::File>&, CFPlacesOpenFlags>
    signalOpenLocation();

    /**
     * 槽函数类型：<br>
     *    void on_my_xx(CFPlacesSidebar param1);
     *
     * @param [in] param1 more_text
     */
    sigc::signal<void, CFPlacesOpenFlags> signalShowOtherLocationsWithFlags();

    /**
     * 槽函数类型：<br>
     *    void on_my_xx(CFPlacesSidebar param1);
     *
     * @param [in] param1 more_text
     */
    sigc::signal<void,
                 CFMenu*,
                 const Glib::RefPtr<Gio::File>&,
                 const Glib::RefPtr<Gio::Volume>&>
    signalPolulatePopup();

    /**
     * 槽函数类型：<br>
     *    void on_my_xx(CFPlacesSidebar param1);
     *
     */
    sigc::signal<void> signalShowConnectToServer();

    /**
     * 槽函数类型：<br>
     *    void on_my_xx(CFPlacesSidebar param1);
     *
     * @param [in] param1 more_text
     */
    sigc::signal<void, const Glib::ustring&, const Glib::ustring&>
    signalShowErrorMessage();

    /**
     * 槽函数类型：<br>
     *    void on_my_xx(CFPlacesSidebar param1);
     *
     * @param [in] param1 more_text
     */
    sigc::signal<void> signalShowEnterLocation();

    /**
     * 槽函数类型：<br>
     *    void on_my_xx(CFPlacesSidebar param1);
     *
     * @param [in] param1 more_text
     */
    sigc::signal<void> signalShowOtherLocations();

    /**
     * 槽函数类型：<br>
     *    void on_my_xx(CFPlacesSidebar param1);
     *
     * @param [in] param1 more_text
     */
    sigc::signal<void, const Glib::RefPtr<Gio::MountOperation>&> signalMount();

    /**
     * 槽函数类型：<br>
     *    void on_my_xx(CFPlacesSidebar param1);
     *
     * @param [in] param1 more_text
     */
    sigc::signal<void, const Glib::RefPtr<Gio::MountOperation>&>
    signalUnmount();

protected:
    virtual void onRealObjectInit();

    virtual void onRealObjectFinalize();

    int onDragActionRequestd(
        const Glib::RefPtr<Gdk::DragContext>& context,
        const Glib::RefPtr<Gio::File>& dest_file,
        const std::vector<Glib::RefPtr<Gio::File>>& source_file_list);

    void onDragPerformDrop(
        const Glib::RefPtr<Gio::File>& dest_file,
        const std::vector<Glib::RefPtr<Gio::File>>& source_file_list,
        int action);

    int onDragActionAsk(int actions);

    void onShowConnectToServer();

    void onShowErrorMessage(const Glib::ustring& primary,
                            const Glib::ustring& secondary);

    void onShowEnterLocation();

    void onShowOtherLocations();

    void onMount(const Glib::RefPtr<Gio::MountOperation>& mount_operation);

    void onUnmount(const Glib::RefPtr<Gio::MountOperation>& mount_operation);

private:
    sigc::signal<int,
                 const Glib::RefPtr<Gdk::DragContext>&,
                 const Glib::RefPtr<Gio::File>&,
                 const std::vector<Glib::RefPtr<Gio::File>>&>
        m_dragActionRequestedSignal;

    sigc::signal<void,
                 const Glib::RefPtr<Gio::File>&,
                 const std::vector<Glib::RefPtr<Gio::File>>&,
                 int>
        m_dragPerformDropSignal;

    sigc::signal<int, int> m_dragActionAskSignal;

    sigc::signal<void> m_showConnectToServerSignal;

    sigc::signal<void, const Glib::ustring&, const Glib::ustring&>
        m_showErrorMessageSignal;

    sigc::signal<void> m_showEnterLocationSignal;

    sigc::signal<void> m_showOtherLocationsSignal;

    sigc::signal<void, const Glib::RefPtr<Gio::MountOperation>&> m_mountSignal;

    sigc::signal<void, const Glib::RefPtr<Gio::MountOperation>&>
        m_unmountSignal;

    // for CFPlacesSidebarPriv
    CFPlacesSidebarPriv* m_priv;

    sigc::signal<void, const Glib::RefPtr<Gio::File>&, CFPlacesOpenFlags>
        m_openLocationSignal;

    sigc::signal<void, CFPlacesOpenFlags> m_showOtherLocationsWithFlagsSignal;

    sigc::signal<void,
                 CFMenu*,
                 const Glib::RefPtr<Gio::File>&,
                 const Glib::RefPtr<Gio::Volume>&>
        m_polulatePopupSignal;
};

CF_NAMESPACE_END

#endif  // __CF_PLACESSIDEBAR_H__
