// Copyright 2017, CDOS.
// All rights reserved.
//
// Author: CDOS Support Team <support_os@sari.ac.cn>
//
// A CFToggleButton will remain 'pressed-in' when clicked.
/**
 * \file
 * 开关按钮
 */
#ifndef __CF_TOGGLEBUTTON_H__
#define __CF_TOGGLEBUTTON_H__

#include "Gui/CFButton.h"

CF_NAMESPACE_BEGIN

/**
 * \brief CFToggleButton 开关按钮
 *
 * 直接继承自 CFButton
 */
class CFToggleButton : public CFButton {
public:
    /**
     * 默认构造函数
     *
     * 创建一个空的开关按钮，可以添加一个 CFImage 或者 CFBox 控件。
     *
     * 如果希望向其中添加一个 CFLabel， 可以使用
     * CFToggleButton(const Glib::ustring& label, bool mnemonic = false);
     *
     * 直接代替
     */
    CFToggleButton();

    /**
     * 构造函数
     *
     * 构造一个包含标签的开关按钮，可以选择是否使用助记符。
     * 如果使用助记符，使用下划线加字母，如”_Open”，按ALT+O可激活这个按钮
     *
     * @param [in] label 用于设置按钮的标签
     * @param [in] mnemonic 是否使用助记符，默认 false
     */
    CFToggleButton(const Glib::ustring& label, bool mnemonic = false);

    /**
     * 析构函数
     * 在 CFToggleButton 被销毁时会被自动调用。
     * 在析构函数中进行内存释放等操作。
     */
    virtual ~CFToggleButton();

    /**
     * 设置按钮模式
     *
     * 设置按钮是否分指示符、标签模式展示。可以在 CFButton 或者 CFRadioButton
     * 中，设置 draw_indicator = false, 来让按钮看起来想普通按钮。
     *
     * 这个函数只能作用于 CFRadioButton 类和 CFToggleButton 类，
     * 而不能作用于自己。
     *
     * @param [in] draw_indicator bool 类型，默认为true。
     */
    void setMode(bool draw_indicator = true);

    /**
     * 获取按钮模式
     *
     * 获取按钮是否是指示符、标签分离的显示模式
     *
     * @return 一个布尔值
     */
    bool getMode();

    /**
     * 设置按钮是否激活
     *
     * 设置按钮的激活状态。
     * 如果想 CFToggleButton 为”pressed in“状态，
     * 设置为 false 让其失效。
     *
     * 这个动作会发送 CFButton::signal_clicked() 和
     * CFToggleButton::signalToggled() 信号。
     *
     * @param [in] is_active true激活, false不激活
     */
    void setActive(bool is_active = true);

    /**
     * 获取按钮的状态
     *
     * 获取按钮当前的激活状态。
     *
     *
     * @return 一个布尔值
     */
    bool getActive();

    /**
     * 发送 CFButton::signalToggled() 信号
     *
     * 如果没有理由，不要调用此程序。
     */
    virtual void toggled();

    /**
     * 槽函数类型：<br>
     *    void onMyToggled();
     *
     *  如果您希望在 CFToggleButton 的状态更改时执行操作，应该连接此信号。
     */
    sigc::signal<void> signalToggled();

private:
    virtual void onRealObjectInit();
    virtual void onRealObjectFinalize();
    sigc::signal<void> m_toggleButtonSignal;
};

CF_NAMESPACE_END

#endif  // __CF_TOGGLEBUTTON_H__
