#ifndef __CF_WIDGET_H__
#define __CF_WIDGET_H__

#include <Core/CFDefines.h>
#include <Core/CFObject.h>
#include <Core/CFTypes.h>
#include <Gui/CFEnums.h>
#include <cairomm/cairomm.h>
#include <gdkmm.h>
#include <sigc++/sigc++.h>

CF_NAMESPACE_BEGIN

/**
 * 所有可视控件的基类，抽象类。
 *
 * 因为该类为所有可视控件的基类，所以包含了所有可视控件类的通用属性<br>
 * 与函数。本类为抽象类，无法被直接实例化。
 * 
 * 可视控件的一个重要部分是 *_event 信号以及所有控件需要用到的虚函数。<br>
 * 这些信号是从Gdk或XLib中发上来的; 可以通过继承并重写这些虚函数来跟踪<br>
 * 控件的对应的事件与信号。需要注意的是：需要监听某个event， 必须先设置<br>
 * 对应的event mask。
 */
class CFWidget : public CFObject {
public:

    /**
     * 析构函数
     *
     * 对象被销毁的时候会被自动调用。
     */
    virtual ~CFWidget();

    /**
     * 设置控件的事件监听掩码。
     *
     * 事件监听掩码决定了控件会接收到哪些事件消息。<br>
     * 每个控件都可能有不同的默认事件监听掩码， 而<br>
     * 修改事件监听掩码可能会干扰控件的功能，所以请<br>
     * 小心使用。<br>
     * 该函数需要在控件未执行realized时被调用。可以<br>
     * 若控件已经执行完realized过程或想要保持已存在<br>
     * 的掩码， 请考虑使用控件的 add_events()。
     *
     * 该函数对于无窗口的控件无效， 因此这些控件若想要<br>
     * 监听/收取事件， 则需要将它们放到 CFEventBox 中,<br>
     * 并监听/收取 CFEventBox 的事件。
     *
     * @param[in] events 需要监听/收取的事件掩码。
     */
    void setEvents (Gdk::EventMask events);

    /**
     * 将 events 位字段添加到控件的事件监听掩码中。
     *
     * @param[in] events 需要监听的事件掩码位字段。
     * @see setEvents()
     */
    void addEvents (Gdk::EventMask events);

    /**
     * 返回当前给控件分配的宽度。
     *
     * 该函数一般使用在 signalDraw() 的槽函数中。
     *
     * @return 控件的宽度。
     */
    int	getAllocatedWidth();

    /**
     * 返回当前给控件分配的高度。
     *
     * 该函数一般使用在 signalDraw() 的槽函数中。
     *
     * @return 控件的高度。
     */
    int	getAllocatedHeight();

    /**
     * 设置控件的最小尺寸。
     *
     * 可以使用该函数设置控件最小尺寸。
     *
     * 大部分情况， 使用 CFWindow::setDefaultSize() 设置顶层窗口的大小更好; <br>
     * 设置其默认尺寸仍然允许缩小窗口。 使用该函数可以强制CFWindow 缩放时不会小于<br>
     * width 和 height所指定的尺寸。当处理窗口尺寸时， CFWindow::setgeometryHints()<br>
     * 会更好用。
     *
     * 该函数参数指定了控件的最小尺寸。不过在一些特殊的情况，控件可能会被分配小于该尺寸， 但<br>
     * 在大部分情况下， 控件分配的尺寸会比该尺寸大。
     *
     * @param[in] width 控件的最小宽度， -1 则取系统默认的最小宽度。
     * @param[in] height 控件的最小高度， -1 则取系统默认的最小高度。
     */
    void setSizeRequest(int width=-1, int height=-1);

    /**
     * 请求重新描画控件
     *
     * 与调用 queueDrawArea() 来请求描画控件整个区域的效果一样。
     */
    void queueDraw();

    /**
     * 请求重新描画的指定区域
     *
     * 重新描画以(x, y)坐标起始，宽度为width, 高度为height所组成的矩形区域。
     */
    void queueDrawArea(int x, int y, int width, int height);

    /**
     * 将控件从当前的容器中移动到 newParent 指定的容器中。
     *
     * @param[in] new_parent 一个CFContainer类型的目标容器。
     */
    void reparent(CFWidget& new_parent);

    /**
     * 指定控件是否可以获取输入焦点。
     *
     * @param[in] can_focus 控件是否可以拥有焦点。
     * @see grabFocus()
     */
    void setCanFocus(bool can_focus = true);

    /**
     * 获取控件是否可以拥有输入焦点。
     *
     * @return true 控件可以拥有焦点。
     * @return false 控件不可以拥有焦点。
     * @see setCanFocus()
     */
    bool getCanFocus();

    /**
     * 判断控件是否拥有全局输入焦点。
     *
     * @return true 控件取得全局输入焦点。
     * @return false 控件未取得全局输入焦点。
     * @see isFocus()
     */
    bool hasFocus();

    /**
     * 判断控件是否当前拥有焦点的控件。
     *
     * @return true 本控件为当前拥有焦点的控件。
     * @return false 本控件非当前拥有焦点的控件。
     */
    bool isFocus();

    /**
     * 控件获取键盘输入焦点。
     *
     * 该控件必须是可获得焦点的控件， 例如：CFEntry等; 类似 CFFrame 的控件无效。
     *
     * 该函数的调用需要控件执行完 realized 和 mapped 过程。<br>
     * 在控件创建完成立即使用本函数获取焦点可能会失败并导致严重的警告。
     */
    void grabFocus ();

    /**
     * 判断控件是否为容器。
     *
     * 只有直接或间接继承自 CFContainer 的类型，该函数会返回true, 其它控件则返回false。
     *
     * @return true 控件为容器对象。
     * @return false 控件不是容器对象。
     */
    virtual bool isContainer();

    /**
     * 设置控件的可见状态。
     *
     * @note 设置为true， 不意味着控件会立刻可见，参照 getVisible(). <br>
     * 该函数会调用 show() 或 hide() ， 但可以更好的在控件依赖某些条件才显示的情况下使用。
     * @param[in] visible 控件是否将被显示。
     */
    void setVisible(bool visible = true);

    /**
     * 判断本控件是否为可见状态。
     *
     * 如果想获取控件的父容器是否也是可见状态， 请使用 isVisible() 来替代。<br>
     * 该函数不会检查控件是否被遮挡。
     *
     * @return true 控件为可见状态。
     * @return false 控件为不可见状态。
     * @see setVisible()
     */
    bool getVisible();

    /**
     * 判断本控件以及控件所在容器(直到顶层容器)是否为可见状态。
     *
     * 该函数不会检查控件是否被遮挡。
     *
     * @return true 控件及其所在容器均为可见状态。
     * @return false 控件或其所在容器为不可见状态。
     * @see getVisible(), setVisible()
     */
    bool isVisible();

    /**
     * 显示控件。
     *
     * 任何控件不设为显示状态均不会被显示在屏幕上。如果想要显示容器控件中的所有控件<br>
     * 调用容器控件的 showAll() 会更方便。<br>
     *
     * 在显示到屏幕之前， 必须要调用控件所在容器的 show() 方法。
     *
     * 当最顶层容器显示出来时，会立即执行 realized 和 mapped过程，然后才会执行其它控件<br>
     * 的 realized 和 mapped 过程。
     */
    void show();

    /**
     * 显示控件以及控件所容纳的所有控件(包括所容纳容器控件的所有子控件)
     *
     */
    void showAll();

    /**
     * 隐藏控件
     *
     * 与 show() 的效果相反， 能将控件隐藏(对用户不可见)
     */
    void hide();

    /**
     * 设置控件的名称，以便于使用CSS文件来控制样式。
     *
     * @param[in] name 控件的名称。
     */
    void setName(const Glib::ustring& name);

    /**
     * 设置控件横向对齐方式。
     *
     * @param[in] align 横向对齐方式。
     */
    void setHalign(CFAlign align);

    /**
     * 获取横向对齐方式。
     *
     * @return 横向对齐方式。
     */
    CFAlign getHalign();

    /**
     * 设置控件纵向对齐方式。
     *
     * @param[in] align 纵向对齐方式。
     */
    void setValign(CFAlign align);

    /**
     * 获取纵向对齐方式。
     *
     * @return 纵向对齐方式。
     */
    CFAlign getValign();

    /**
     * 设置提示信息的内容。
     *
     * @param[in] text 提示信息的内容。
     */
    void setTooltipText(const Glib::ustring& text);

    /**
     * 获取当前控件提示信息的内容。
     *
     * @return 当前控件提示信息的内容。
     */
    Glib::ustring getTooltipText();

    /**
     * 设置提示信息的内容。
     *
     * 该函数markup参数，支持使用 Pango 文本标记语言格式的字符串。
     *
     * @param[in] markup 提示信息的内容， 支持使用 Pango 文本标记语言格式的字符串。
     */
    void setTooltipMarkup(const Glib::ustring& markup);

    /**
     * 获取当前控件提示信息的内容。
     *
     * @return 提示信息的内容， 可能包含 Pango 文本标记语言格式的字符串。
     */
    Glib::ustring getTooltipMarkup();

    /**
     * 设置是否使用提示信息。
     *
     * @param[in] has_tooltip 是否使用提示信息。
     */
    void setHasTooltip(bool has_tooltip = true);

    /**
     * 获取是否使用提示信息。
     *
     * @return 是否使用提示信息。
     */
    bool getHasTooltip();

    /**
     * 设置控件是否可用。
     *
     * 设置控件的可用状态， 若设置为不可用状态，则控件无法接收到各种事件消息， 并被置灰。<br>
     *
     * @param[in] sensitive 可用状态。
     */
    void setSensitive(bool sensitive = true);

    /**
     * 获取控件可用状态
     *
     * 本函数只会获取控件的可用状态，不会判断所在容器的可用状态，<br>
     * 如果想获取控件的父容器是否也是可见状态， 请使用 isSensitive() 来替代。<br>
     * 
     * @return true 控件为可用状态。
     * @return false 控件为不可用状态。
     */
    bool getSensitive();

    /**
     * 获取控件及其所在容器控件的可用状态
     *
     * @return true 控件及所在容器为可用状态。
     * @return false 控件或所在容器为不可用状态。
     */
    bool isSensitive();

    /**
     * 设置控件的左边距
     *
     * @param[in] margin 左边距值，以像素为单位。
     */
    void setMarginLeft(int margin);

    /**
     * 获取控件的左边距
     *
     * @return 返回控件左边距值，以像素为单位。
     */
    int getMarginLeft();

    /**
     * 设置控件的右边距
     *
     * @param[in] margin 右边距值，以像素为单位。
     */
    void setMarginRight(int margin);

    /**
     * 获取控件的右边距
     *
     * @return 返回控件右边距值，以像素为单位。
     */
    int getMarginRight();

    /**
     * 设置控件的上边距
     *
     * @param[in] margin 上边距值，以像素为单位。
     */
    void setMarginTop(int margin);

    /**
     * 获取控件的上边距
     *
     * @return 返回控件上边距值，以像素为单位。
     */
    int getMarginTop();

    /**
     * 设置控件的下边距
     *
     * @param[in] margin 下边距值，以像素为单位。
     */
    void setMarginBottom(int margin);

    /**
     * 获取控件的下边距
     *
     * @return 返回控件下边距值，以像素为单位。
     */
    int getMarginBottom();

    virtual void addCssStylePath(const Glib::ustring& path, CFUInt priority);
    virtual void addCssStyleData(const Glib::ustring& data, CFUInt priority);

    /**
     * 通过类名设置样式
     *
     * @name: 类名的应用，通常在css文件中以'.'开头；
     *
     * 为控件设置样式，该函数仅对当前主题css文件样式中的类有效。
     *
     */
    virtual void addCssStyleName(const Glib::ustring& name);

    /**
     * 通过自定义css样式文件路径设置样式
     *
     * @param[in] css_file_path css文件的绝对路径
     * @param[in] name 既可以是类选择器（在css中以'.'开头），也可以是id选择器（在css中以'#'开头）
     * @param[in] priority 显示权重
     *
     * 为控件设置样式，该函数无视当前主题样式，可以自定义文件路径。
     */
    virtual void addCssStyleFromPath(const Glib::ustring& css_file_path,
                                     const Glib::ustring& name,
                                     CFUInt priority);

    /**
     * 查看样式列表
     *
     */
    virtual void listCssStyle();

    /**
     * 获取控件样式列表
     *
     * @return: 返回一个std::vector类型的样式列表
     */
    virtual std::vector<Glib::ustring> getCssStyleVector();

    /**
     * 通过样式名移除样式
     *
     * @param[in] name 样式名
     */
    virtual void removeCssStyle(const Glib::ustring& name);

    Glib::RefPtr< Gdk::Window > getWindow();

public:
    sigc::signal< bool, const ::Cairo::RefPtr< ::Cairo::Context >& > 
        signalDraw();
    sigc::signal< bool, GdkEvent* > signalEvent();
    sigc::signal< bool, GdkEventButton* > signalButtonPressEvent();
    sigc::signal< bool, GdkEventButton* > signalButtonReleaseEvent();
    sigc::signal< bool, GdkEventAny* > signalDeleteEvent();

    sigc::signal< bool, GdkEventKey* > signalKeyPressEvent();
    sigc::signal< bool, GdkEventKey* > signalKeyReleaseEvent();

    sigc::signal< void > signalShow();

protected:

    virtual void onRealObjectInit();
    virtual void onRealObjectFinalize();

    virtual bool onDraw(const ::Cairo::RefPtr< ::Cairo::Context > & cr);
    virtual bool onEvent(GdkEvent* event);
    virtual bool onButtonPressEvent(GdkEventButton* event);
    virtual bool onButtonReleaseEvent(GdkEventButton* event);
    virtual bool onDeleteEvent(GdkEventAny* event);

    virtual bool onKeyPressEvent(GdkEventKey* event);
    virtual bool onKeyReleaseEvent(GdkEventKey* event);

    virtual void onShow();

    CFWidget();

private:
    sigc::signal< bool, const ::Cairo::RefPtr< ::Cairo::Context >& > m_drawSignal;
    sigc::signal< bool, GdkEvent* > m_eventSignal;
    sigc::signal< bool, GdkEventButton* > m_buttonPressEventSignal;
    sigc::signal< bool, GdkEventButton* > m_buttonReleaseEventSignal;
    sigc::signal< bool, GdkEventAny* > m_deleteEventSignal;

    sigc::signal< bool, GdkEventKey* > m_keyPressEventSignal;
    sigc::signal< bool, GdkEventKey* > m_keyReleaseEventSignal;

    sigc::signal< void > m_showSignal;
};

CF_NAMESPACE_END

#endif // __CF_WIDGET_H__
